@@ -34,7 +34,8 @@
 import sys
 
 from logilab.common import testlib
-from logilab.astng import builder, nodes, NotFoundError
+from logilab.astng.exceptions import ASTNGBuildingException, NotFoundError
+from logilab.astng import builder, nodes
 from logilab.astng.as_string import as_string
 
 from data import module as test_module
@@ -45,6 +46,56 @@ DATA = join(dirname(abspath(__file__)), 'data')
 
 abuilder = builder.ASTNGBuilder()
 
+class AsString(testlib.TestCase):
+
+    def test_varargs_kwargs_as_string(self):
+        ast = abuilder.string_build( 'raise_string(*args, **kwargs)').body[0]
+        self.assertEqual(as_string(ast), 'raise_string(*args, **kwargs)')
+
+    def test_module_as_string(self):
+        """check as_string on a whole module prepared to be returned identically
+        """
+        data = open(join(DATA, 'module.py')).read()
+        self.assertMultiLineEqual(as_string(MODULE), data)
+        data = open(join(DATA, 'module2.py')).read()
+        self.assertMultiLineEqual(as_string(MODULE2), data)
+
+    def test_2_7_as_string(self):
+        """check as_string for python syntax >= 2.7"""
+        if sys.version_info < (2, 7):
+            self.skipTest("test python >= 2.7 specific")
+        code = '''one_two = {1, 2}
+b = {v: k for (k, v) in enumerate('string')}
+cdd = {k for k in b}\n\n'''
+        ast = abuilder.string_build(code)
+        self.assertMultiLineEqual(as_string(ast), code)
+
+    def test_3k_as_string(self):
+        """check as_string for python 3k syntax"""
+        if sys.version_info < (3, 0):
+            self.skipTest("test python 3k specific")
+        code = '''print()
+
+def function(var):
+    nonlocal counter
+    try:
+        hello
+    except NameError as nexc:
+        (*hell, o) = b'hello'
+        raise AttributeError from nexc
+\n'''
+        # TODO : annotations and keywords for class definition are not yet implemented
+        _todo = '''
+def function(var:int):
+    nonlocal counter
+
+class Language(metaclass=Natural):
+    """natural language"""
+        '''
+        ast = abuilder.string_build(code)
+        self.assertEqual(as_string(ast), code)
+
+
 class _NodeTC(testlib.TestCase):
     """test transformation of If Node"""
     CODE = None
@@ -214,59 +265,18 @@ class ImportNodeTC(testlib.TestCase):
         self.assertRaises(NotFoundError, imp_.real_name, 'data')
 
     def test_as_string(self):
-        
         ast = MODULE['modutils']
         self.assertEqual(as_string(ast), "from logilab.common import modutils")
         ast = MODULE['spawn']
         self.assertEqual(as_string(ast), "from logilab.common.shellutils import Execute as spawn")
         ast = MODULE['os']
         self.assertEqual(as_string(ast), "import os.path")
-        ast = abuilder.string_build( 'raise_string(*args, **kwargs)').body[0]
-        self.assertEqual(as_string(ast), 'raise_string(*args, **kwargs)')
-
-    def test_module_as_string(self):
-        """check as_string on a whole module prepared to be returned identically
-        """
-        data = open(join(DATA, 'module.py')).read()
-        self.assertMultiLineEqual(as_string(MODULE), data)
-        data = open(join(DATA, 'module2.py')).read()
-        self.assertMultiLineEqual(as_string(MODULE2), data)
-
-    def test_2_7_as_string(self):
-        """check as_string for python syntax >= 2.7"""
-        if sys.version_info < (2, 7):
-            self.skipTest("test python >= 2.7 specific")
-        code = '''one_two = {1, 2}
-b = {v: k for (k, v) in enumerate('string')}
-cdd = {k for k in b}\n\n'''
+        code = """from . import here
+from .. import door
+from .store import bread
+from ..cave import wine\n\n"""
         ast = abuilder.string_build(code)
-        self.assertMultiLineEqual(as_string(ast), code)
-
-    def test_3k_as_string(self):
-        """check as_string for python 3k syntax"""
-        if sys.version_info < (3, 0):
-            self.skipTest("test python 3k specific")
-        code = '''print()
-
-def function(var):
-    nonlocal counter
-    try:
-        hello
-    except NameError as nexc:
-        (*hell, o) = b'hello'
-        raise AttributeError from nexc
-\n'''
-        # TODO : annotations and keywords for class definition are not yet implemented
-        _todo = '''
-def function(var:int):
-    nonlocal counter
-
-class Language(metaclass=Natural):
-    """natural language"""
-        '''
-        ast = abuilder.string_build(code)
-        self.assertEqual(as_string(ast), code)
-
+        self.assertMultiLineEqual(ast.as_string(), code)
 
 class CmpNodeTC(testlib.TestCase):
     def test_as_string(self):
@@ -306,6 +316,27 @@ class ConstNodeTC(testlib.TestCase):
         self._test(u'a')
 
 
+class NameNodeTC(testlib.TestCase):
+    def test_assign_to_True(self):
+        """test that True and False assignements don't crash"""
+        code = """True = False
+def hello(False):
+    pass
+del True
+    """
+        if sys.version_info >= (3, 0):
+            self.assertRaises(SyntaxError,#might become ASTNGBuildingException
+                              abuilder.string_build, code)
+        else:
+            ast = abuilder.string_build(code)
+            ass_true = ast['True']
+            self.assertIsInstance(ass_true, nodes.AssName)
+            self.assertEqual(ass_true.name, "True")
+            del_true = ast.body[2].targets[0]
+            self.assertIsInstance(del_true, nodes.DelName)
+            self.assertEqual(del_true.name, "True")
+            
+
 class ArgumentsNodeTC(testlib.TestCase):
     def test_linenumbering(self):
         ast = abuilder.string_build('''
@@ -314,12 +345,16 @@ def func(a,
 x = lambda x: None
         ''')
         self.assertEqual(ast['func'].args.fromlineno, 2)
-        self.assertEqual(ast['func'].args.tolineno, 3)
         self.failIf(ast['func'].args.is_statement)
         xlambda = ast['x'].infer().next()
         self.assertEqual(xlambda.args.fromlineno, 4)
         self.assertEqual(xlambda.args.tolineno, 4)
         self.failIf(xlambda.args.is_statement)
+        if sys.version_info < (3, 0):
+            self.assertEqual(ast['func'].args.tolineno, 3)
+        else:
+            self.skipTest('FIXME  http://bugs.python.org/issue10445 '
+                          '(no line number on function args)')
 
 
 class SliceNodeTC(testlib.TestCase):
